//
// Gertbot project
// Serial routines
// I did not know where to place these so they ended up in this file. 
// They are 'serial' communication routines
// but using the standard uart_read/uart_write calls 
// 

#include "gertbot.h"

#ifdef _WIN32

#else
#include <unistd.h> // for usleep(x)

#endif


int log_show_reply = 0;

//
// This routine tries to find all Gertbot boards
// It is often the first one which uses the serial channel
// so its has code to cope with a 'bad' link
//
int find_board(int b,int *ver_ms,int *ver_ls)
{ unsigned char rx_data[64];
  unsigned char tx_data[16];
  int try_count,rec_count;
  unsigned char board_code;
  try_count=0;
  // As this is likely the first attempt to communicate
  // add plenty of stop codes on both sides of the command
  board_code = b<<2;
  for (b=0; b<8; b++)
    tx_data[b] = CMD_STOP_VAL;
  tx_data[b++] = CMD_START_VAL;
  tx_data[b++] = CMD_VERSION;
  tx_data[b++] = board_code;
  for ( ; b<16; b++)
    tx_data[b] = CMD_STOP_VAL;
  while (try_count<4)
  { // send command & wait for reply 
    write_uart(tx_data,16);
    rec_count = read_uart(rx_data,4);
    
    // Syncing so have have to be strict 
    // refuse if we do not get perfect data
    if (rec_count==4 && rx_data[0]==board_code && rx_data[1]==CMD_VERSION)
    { // assume success
      *ver_ms = rx_data[2];
      *ver_ls = rx_data[3];        
      return 1;
    }
    try_count++;
  } // try several times
  read_uart(rx_data,16); // Drop any read buffer remainders  
  return 0;
} // find_board

//
// Wait for reply of a previous sent command
//
// return 1 on OK
// return 0 on failure
//
// If OK *data holds received data
//
int get_reply(unsigned char *data,  // received re-ply
              int byte_count,       // number of bytes we expect
              unsigned char id,     // Board id
              unsigned char command)// command sent
{ int rec_count,s,skip;
  int match,new_rec;
  unsigned char dummy[32];
  SLEEP_MS(10); // give a bit of time to reply 
  rec_count = read_uart(data,byte_count);
//  printf("#=%d 0=%d 1=%d 2=%d\n",rec_count,data[0],data[1],data[2]);
  if (log_show_reply)
  {
    if (rec_count==byte_count && data[0]==id && data[1]==command)
    { char buffer[1024],*bp;
      bp = buffer;
      *bp++ = '<';
      for (s=0; s<rec_count; s++)
      { sprintf(bp,"%02X ",data[s]);
        bp += 3;
      }
      gertbot_pnt->add_log(buffer); 
      return 1;
    }
  }
    
  // We seems to have a communication problem
  // Desparate try to recover in some way
  // Try to discard any rubbish at begin
  skip=0; match=0;
  while (!match && skip<rec_count-1)
  { match = (data[skip]==id && data[skip+1]==command);
    if (!match) 
      skip++;
  }

  if (match)
  { // We seem to have found the start of the reply
    // move the data to the start of the buffer
    s = 0;
    while (s<rec_count)
    { data[s] = data[s+skip];
      s++;
    }
    // Now try to get the missing tail bytes
    new_rec = read_uart(data+s,skip);
    if (new_rec==skip)
      return 1;
  }

  // Don't know anymore what to do 
  // Try at least clear input buffer
  read_uart(dummy,32);
  return 0;
} // get_reply

// Send one byte, also echo to log window 
void uart_tx(byte b0)
{ unsigned char uart_tx_data[3];
  QString str; 
  str.sprintf("0x%02X",b0);
  gertbot_pnt->add_log(str); 
  uart_tx_data[0] = 0xA0;
  uart_tx_data[1] = b0;
  uart_tx_data[2] = 0x50;
  if (uart_opened)
    write_uart(uart_tx_data,3);
}
void uart_tx(byte b0 ,byte b1) 
{ unsigned char uart_tx_data[4];
  QString str; 
  str.sprintf("0x%02X 0x%02X",b0,b1); 
  gertbot_pnt->add_log(str); 
  uart_tx_data[0] = 0xA0;
  uart_tx_data[1] = b0;
  uart_tx_data[2] = b1;
  uart_tx_data[3] = 0x50;
  if (uart_opened)
    write_uart((unsigned char *)uart_tx_data,4);
}
void uart_tx(byte b0 ,byte b1 ,byte b2) 
{ unsigned char uart_tx_data[5];
  QString str; 
  str.sprintf("0x%02X 0x%02X 0x%02X",b0,b1,b2);
  gertbot_pnt->add_log(str); 
  uart_tx_data[0] = 0xA0;
  uart_tx_data[1] = b0;
  uart_tx_data[2] = b1;
  uart_tx_data[3] = b2;
  uart_tx_data[4] = 0x50;
  if (uart_opened)
    write_uart((unsigned char *)uart_tx_data,5);
}
void uart_tx(byte b0 ,byte b1 ,byte b2, byte b3) 
{ unsigned char uart_tx_data[6];
  QString str; 
  str.sprintf("0x%02X 0x%02X 0x%02X 0x%02X",b0,b1,b2,b3);
  gertbot_pnt->add_log(str); 
  uart_tx_data[0] = 0xA0;
  uart_tx_data[1] = b0;
  uart_tx_data[2] = b1;
  uart_tx_data[3] = b2;
  uart_tx_data[4] = b3;
  uart_tx_data[5] = 0x50;
  if (uart_opened)
    write_uart((unsigned char *)uart_tx_data,6);
}
void uart_tx(byte b0 ,byte b1 ,byte b2, byte b3, byte b4) 
{ unsigned char uart_tx_data[7];
  QString str; 
  str.sprintf("0x%02X 0x%02X 0x%02X 0x%02X 0x%02X",b0,b1,b2,b3,b4);
  gertbot_pnt->add_log(str); 
  uart_tx_data[0] = 0xA0;
  uart_tx_data[1] = b0;
  uart_tx_data[2] = b1;
  uart_tx_data[3] = b2;
  uart_tx_data[4] = b3;
  uart_tx_data[5] = b4;
  uart_tx_data[6] = 0x50;
  if (uart_opened)
    write_uart((unsigned char *)uart_tx_data,7);
}

void uart_tx(byte *buf,int length,int pad)
{ byte tx_buf[64]; // sloppy. I know 
  int i,b;
  QString str1,str2;
  if (length+pad>63)
    exit(111);
  b=0;
  tx_buf[b++] = 0xA0;
  for (i=0; i<length; i++)
  { str2.sprintf("0x%02X ",*buf);
    str1 += str2;
    tx_buf[b++]=*buf++;
  }
  gertbot_pnt->add_log(str1); 
  tx_buf[b++]=0x50;
  for (i=0; i<pad; i++)
    tx_buf[b++]=0x50;
  write_uart(tx_buf,b);
} // uart_tx buffer plus pads 

/*
void uart_tx(byte b0 ,byte b1 ,uint32_t i32) 
{ QString str; str.sprintf("0x%02X 0x%02X 0x%02X 0x%02X %02X",
   b0,b1,(i32>>16)&0xFF,(i32>>8)&0xFF,i32&0xFF); 
   gertbot_pnt->add_log(str); 
}
void uart_tx(byte b0 ,byte b1 ,uint16_t i16) 
{ QString str; str.sprintf("0x%02X 0x%02X 0x%02X 0x%02X",
   b0,b1,(i16>>8)&0xFF,i16&0xFF); 
   gertbot_pnt->add_log(str); 
}
*/
void status(QString str,int append)
{ QString total;
  if (append)
    total += str;
  else
  { total += str;
    gertbot_pnt->add_log(total);
    total = "";
  } 
}


